#include <stdint.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/eeprom.h>
#include <util/delay.h>
#include "hardware.h"

static volatile uint8_t g_delay;

static void uartInit(void)
{
	// 115200 baud at 18.432 MHz
	UBRRH = 0;
	UBRRL = 9;
	
	// config UART transmitter and receiver for 8N1 and active it
	UCSRB = _BV(RXEN) | _BV(TXEN);
	UCSRC = _BV(UCSZ0) | _BV(UCSZ1);
}

static uint8_t uartIsByteReceived(void)
{
	return (UCSRA & _BV(RXC)) > 0;
}

static uint8_t uartGetByte(void)
{
	return UDR;	
}

static uint8_t uartIsSendReady(void)
{
	return (UCSRA & _BV(UDRE)) > 0;
}

static void uartSendByte(uint8_t b)
{
	UDR = b;
}

// config timer for 1 kHz interrupt
static void timerInit(void)
{
	TCCR1A = 0x00;
	TCCR1B = _BV(WGM12) | _BV(CS11);
	OCR1A = 2303;
	TIMSK = _BV(OCIE1A); 
}

uint8_t getA(void)
{
	return (PINB & _BV(1)) > 0;
}

uint8_t getB(void)
{
	return (PINB & _BV(0)) > 0;
}

ISR(TIMER1_COMPA_vect)
{
	onInterrupt();
	g_delay++;
}

void writeChar(uint8_t c)
{
	while (!uartIsSendReady());
	uartSendByte(c);
}

uint8_t readChar(void)
{
	while (!uartIsByteReceived());
	return uartGetByte();
}

void clearReceiveBuffer(void)
{
	unsigned char dummy;
	while (UCSRA & (_BV(RXC))) dummy = UDR;
}

void initHardware(void)
{
	uartInit();
	timerInit();
	sei();
}

void delayMs(uint16_t ms)
{
	while (ms) {
		uint8_t delay0 = g_delay;
		while (delay0 == g_delay);
		ms--;
	}
}

void disableInterrupts(void)
{
	cli();
}

void enableInterrupts(void)
{
	sei();
}
